function I = painter(vertices, faces, M)

numPixX = 1000;
numPixY = 1000;

numFaces = size(faces,1);
numVertices = size(vertices,1);

I = zeros(numPixY, numPixX, 'uint8');
depth = 10000*ones(numPixY, numPixX);

A = M(1:3,1:3);
Ainv = A^-1;
b = M(1:3,4);

W = -Ainv*b;

%source = [-1;-1;1];
source = -W;
source = source/norm(source);

p = M*[vertices(1,:)';1];
p = p/p(3);
xMin = p(1);
xMax = p(1);
yMin = p(2);
yMax = p(2);
for v = 2:numVertices
    p = M*[vertices(v,:)';1];
    p = p/p(3);
    if(p(1)<xMin)
        xMin = p(1);
    end
    if(p(1)>xMax)
        xMax = p(1);
    end
    if(p(2)<yMin)
        yMin = p(2);
    end
    if(p(2)>yMax)
        yMax = p(2);
    end
end

xScale = floor(min([(numPixX-50)/(xMax-xMin), (numPixY-50)/(yMax-yMin)]));
xOffset = floor(xScale*xMin) - 26;
yScale = xScale;
yOffset = floor(yScale*yMin) - 26;

for f = 1:numFaces
    for i = 1:3
        Ps(1:4,i) = [vertices(faces(f,i),:), 1]';
    end
    ps = M*Ps;
    ps = [xScale*ps(1,:)./ps(3,:) - xOffset; yScale*ps(2,:)./ps(3,:) - yOffset; ones(1,3)];
    xMin = floor(min(ps(1,:)));
    xMax = ceil(max(ps(1,:)));
    yMin = floor(min(ps(2,:)));
    yMax = ceil(max(ps(2,:)));
    
    for i = 1:3
        m(i) = (ps(2,mod(i,3)+1)-ps(2,i))/(ps(1,mod(i,3)+1)-ps(1,i));
        b(i) = ps(2,i) - m(i)*ps(1,i);
        side(i) = sign(ps(2,mod(i-2,3)+1) - m(i)*ps(1,mod(i-2,3)+1) - b(i));
    end
    
    P12 = Ps(1:3,2) - Ps(1:3,1);
    P13 = Ps(1:3,3) - Ps(1:3,1);
    n = cross(P12,P13);
    n = n/norm(n);
    k = dot(source,n);
    reflection = source - 2*k*n;
    if k > 0
        reflection = [0;0;0];
    end
    
    for x = xMin:xMax
        for y = yMin:yMax
            inside = 1;
            for i = 1:3
                inside = inside * (sign(y - m(i)*x - b(i)) == side(i));
            end
            if inside == 1
                Pavg = mean(Ps,2);
                d = norm(Pavg(1:3,1) - W);
                if(d < depth(y,x))
                    depth(y,x) = d;
                    ray = Ainv*[(x+xOffset)/xScale;(y+yOffset)/yScale;1];
                    ray = ray/norm(ray);
                    theta = acos(-dot(reflection,ray));
                    if theta > pi/2
                        light = 0.2;
                    else
                        light = 256*(-0.5*theta + 1);
                    end
                    I(y,x) = light;
                end
            end
        end
    end
    %Display Progress every 10%
    if(mod(f,round(numFaces/10))==0)
        disp([num2str(round(f/numFaces*100)),'%']);
    end
end
